/*
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        28.05.2004
 Description:    Dieses Programm dient als Beispiel zur Ansteuerung einer MMC/SD-Memory-Card.
				 Zum Zugriff auf eine MMC/SD-Karte, mu man nur die Datei mmc.c
				 in sein eigenes Programm einfgen.
*/

#include <main.h>

//Anpassen der Register wenn ein ATMega128 benutzt wird
#if defined (__AVR_ATmega128__)
#	define USR UCSR0A
#	define UCR UCSR0B
#	define UDR UDR0
#	define UBRR UBRR0L
#	define EICR EICRB
#endif

//Anpassen der Register wenn ein ATMega32 benutzt wird
#if defined (__AVR_ATmega32__)
#	define USR UCSRA
#	define UCR UCSRB
#	define UBRR UBRRL
#	define EICR EICRB
#endif

//Anpassen der Register wenn ein ATMega162 benutzt wird
#if defined (__AVR_ATmega162__)
#	define USR UCSR0A
#	define UCR UCSR0B
#	define UBRR UBRR0L
#	define EICR EICRB
#endif

//Der Quarz auf dem Experimentierboard hat eine Frequenz von 7.3728MHz
#define SYSCLK	7372800	//Quarz Frequenz in Hz

//Die Baud_Rate der Seriellen Schnittstelle ist 38400 Baud
#define BAUD_RATE 38400		//Baud Rate fr die Serielle Schnittstelle	

unsigned char Buffer[512];


//Installation der Seriellen Schnittstelle


void IOInit (void)
{
	//Enable TXEN im Register UCR TX-Data Enable
	UCR=(1 << TXEN0);
	//Teiler wird gesetzt 
	UBRR=(SYSCLK / (BAUD_RATE * 16L) - 1);
}


//Routine fr printf
int uart_putchar (char c)
{
	if (c == '\n')
		uart_putchar('\r');
	//Warten solange bis Zeichen gesendet wurde
	loop_until_bit_is_set(USR, UDRE0);
	UDR0 = c;
	return (0);
}

void loadbin(unsigned char * name, unsigned int addr)
{	//Lade Cluster fr das Memoryfile in den Speicher
	unsigned int Cluster = 0;//suche im Root Verzeichnis
	unsigned char Dir_Attrib = 0;
	unsigned long Size = 0;
	unsigned int c;
	if (Search_File(name,&Cluster,&Size,&Dir_Attrib,Buffer) == 1)
		{
		printf("\nLoad %s!\n",name);
                unsigned int b=0;
		c=512;
                while(Size>0)
                        {
                        FPGA_write_addr(addr);
			Read_Memory(Cluster,Buffer,b);
			if (Size<512)
			   c=Size;
			Size-=c;
			b+=1;
			addr+=2;
			}
	        printf("%s ready!\n",name);
		}

}

//Hauptprogramm
int main (void)
{
	//Initzialisierung der seriellen Schnittstelle
	IOInit();
	
	//ffnet einen kanal fr printf
	fdevopen (uart_putchar, NULL, 0);

         //FPGA loeschen
                DDRC|=0xbd;
//                DDRC&=0xfd;
                PORTC=0xf7;  //nConfig Low
                printf ("\n\nErasing FPGA\n");	
                PORTC=0xff;  //nConfig High

	//Initialisierung der MMC/SD-Karte
	printf ("System OK\n");	
	while ( mmc_init() !=0) //ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten
		{
		printf("** No MMC/SD Card found!! **\n");	
//		printf("** Son Mist!! **\n");	
		}
	printf("Found MMC/SD Card!\n");
	
	//unsigned char Buffer[512];
//	char *Buffer;
//	Buffer =(char *)malloc(513);
	Cluster_Data_Store();//laden Cluster OFFSET und Size ins EEPROM
	//Initialisierung der MMC/SD-Karte ENDE!

//	unsigned int tmp;
	
	Read_CSD_MMC (Buffer);
	
//	for (tmp = 0;tmp<16;tmp++)
//		{
//		printf("%x ",Buffer[tmp]);
//		};




	//Ausgabe des Root Directory
	unsigned int Cluster;
	unsigned char Dir_Attrib = 0;
	unsigned long Size = 0;
	unsigned int c;
//	unsigned char d;
//	printf("\nDirectory\n\n");
//	for (unsigned char a = 1;a < 240;a++)
//	{
//		Cluster = Read_Dir_Ent(0,a,&Size,&Dir_Attrib,Buffer);
//			if (Cluster == 0xffff)
//			{
//				break;
//			}
//		tmp = (Size & 0x0000FFFF);
//		printf("Cluster = %4x DirA = %2x Size= %8d FileName = ",Cluster,Dir_Attrib,tmp);
//		printf(Buffer);
//		printf("\n");
//	}
//	printf("\nDirectory Ende\n\n");

	//Lade Cluster fr das Configfile in den Speicher
	Cluster = 0;//suche im Root Verzeichnis
	if (Search_File("onbo.rbf",&Cluster,&Size,&Dir_Attrib,Buffer) == 1)
		{
//                DDRC|=0xbd;
//                DDRC&=0xfd;
//                PORTC=0xf7;  //nConfig Low
//		printf("\nFile Found!!\n\n");
		//Lese File und gibt es auf der seriellen Schnittstelle aus
//                PORTC=0xff;  //nConfig High
                unsigned int b=0;
//                unsigned int as=Size/512;
		c=512;
//		for (int b = 0;b<=as;b++)
                while(Size>0)
                        {
			Read_Config(Cluster,Buffer,b);
//		        tmp = ((Size/512) & 0x0000FFFF);
//	                printf("\nFERTIG!!Lala Size= %8d \n",tmp);
			if (Size<512)
			   c=Size;
			Size-=c;
			b+=1;
			}
      	        printf("\nConfig ready!\n");
		}


        FPGA_write_addr(0x0ffd);       //FPGA Reset auf L

	//Lade Cluster fr das Memoryfile in den Speicher

        loadbin("RISC.bin",0x0800);
//        loadbin("RISC.bin",0x0C00);

        loadbin("BigBoot.bin",0x0AC0);

        loadbin("ROML.bin",0x0900);
//        loadbin("ROML.bin",0x0E00);

        loadbin("tfROM.bin",0x09C0);
	
        loadbin("Floppy.dsk",0x0000);

        loadbin("ROM00.bin",0x0C00);
        loadbin("ROM01.bin",0x0C40);
        loadbin("ROM02.bin",0x0C80);
        loadbin("ROM03.bin",0x0CC0);
        loadbin("ROM04.bin",0x0D00);
        loadbin("ROM05.bin",0x0D40);
        loadbin("ROM06.bin",0x0D80);
        loadbin("ROM07.bin",0x0DC0);
        loadbin("ROM08.bin",0x0E00);
        loadbin("ROM09.bin",0x0E40);
        loadbin("ROM10.bin",0x0E80);
        loadbin("ROM11.bin",0x0EC0);
        loadbin("ROM12.bin",0x0F00);
        loadbin("ROM13.bin",0x0F40);
        loadbin("ROM14.bin",0x0F80);
        loadbin("ROM15.bin",0x0Fc0);


      	printf("\nStart CPC!\n\n");

	//Enable TXEN im Register UCR TX-Data Enable
	UCR&=~(1 << TXEN0);
        FPGA_write_addr(0x0fff);       //FPGA Reset auf H
        DDRB = 0;
        DDRA = 0;
        DDRC = 0;
        DDRD = 0;
        
//	MMC_Write &=~(1<<SPI_Clock); //erzeugt ein Clock Impuls (Low)
//	MMC_Write |=(1<<SPI_Clock); //setzt Clock Impuls wieder auf (High)		
        //Hauptprogramm luft stndig in einer schleife und macht nichts
	while (1)
		{
		}
return (1);
}

