-- CPC ONBO reconfigurable computer
--
-- Copyright (c) 2005 Tobias Gubener
--
-- All rights reserved
--
-- Redistribution and use in source and synthezised forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- Redistributions of source code must retain the above copyright notice,
-- this list of conditions and the following disclaimer.
--
-- Redistributions in synthesized form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
--
-- Neither the name of the author nor the names of other contributors may
-- be used to endorse or promote products derived from this software without
-- specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.
--



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity spi is
port
	(
	clk			: in std_logic;
	scki		: in std_logic;
	fpcs		: in std_logic;
	sdcs		: in std_logic;
	spi			: in std_logic;
	spo			: in std_logic;
	nreset		: in std_logic;
	addr		: out std_logic_vector(19 downto 0);
	data		: out std_logic_vector(7 downto 0);
	ram_wr 		: out std_logic;
	reset		: out std_logic
	);
end;

architecture rtl of spi is

signal shift	: std_logic_vector(14 downto 0);
signal shiftcnt	: std_logic_vector(3 downto 0);
signal addr_cnt	: std_logic_vector(26 downto 0);
signal reseti	: std_logic;
signal ram_wri	: std_logic;
signal ram_wro	: std_logic;
signal sck		: std_logic;


begin
	ram_wr <= ram_wro and reseti;   --write in H
	reset <= not reseti and nreset;
	process (clk) begin
		if (clk'event and clk='1') then
			sck <= scki;
		end if;	
	end process;
	
	process (nreset, sck) begin
		if nreset ='0' then 
			ram_wro <= '0';
			ram_wri <= '0';
		elsif (sck'event and sck='1') then
			ram_wro <= ram_wri;
			if fpcs='0' AND sdcs='0' AND addr_cnt(2 downto 0)=7 then
				ram_wri <= '1';
			elsif addr_cnt(2 downto 0)=2 then
				ram_wri <= '0';
			end if;
		end if;
	end process;

	process (sck, nreset, fpcs) begin
		if nreset ='0' then 
			reseti <= '0';
		elsif fpcs = '1' then
			shiftcnt <= "0000";
		elsif (sck'event and sck='1') then
			if fpcs='0' then
				if sdcs='1' then
					shift <= (shift(13 downto 0)& spi);
					addr_cnt <= shift & spi & "00000000000";
					shiftcnt <= shiftcnt+1;
					if shiftcnt = 15 then
						if (shift & spi) = X"FFF" then
							reseti <= '0';
						elsif (shift & spi) = X"FFD" then
							reseti <= '1';
						end if;	
					end if;
				else
					shift <= (shift(13 downto 0)& spo);
					addr_cnt <= addr_cnt + 1;
					if addr_cnt(2 downto 0)=7 then
						data <= shift(6 downto 0)& spo;
						addr <= addr_cnt(22 downto 3);
					end if;	
				end if;
			end if;
		end if;
	end process;
end;
